﻿using System;
using System.Linq;
using System.Reflection;
using UnityEngine;

namespace UnityEditor
{
    public class GUISkinViewer : EditorWindow
    {
        [MenuItem("Window/Supporting/GUISkin Viewer")]
        public static void Create()
        {
            GetWindow<GUISkinViewer>(true, "GUISkin Viewer", true);
        }

        public GUISkin guiSkin;
        public Vector2 scrollPos;
        public int index = 0;

        public string find = "";

        public GUIStyle[] showStyles;

        public void OnGUI()
        {
            var up = guievent.downKeyIs(KeyCode.UpArrow);
            var down = guievent.downKeyIs(KeyCode.DownArrow);
            using (gui.group_horizontal(css.expand_width(true)))
            {
                using (gui.group_vertical(css.width(300)))
                {
                    gui.textfield(ref find, css.label("Find"));
                    guiSkin = EditorGUILayout.ObjectField("GUISkin", guiSkin, typeof(GUISkin), false) as GUISkin;
                    if (gui.button("Pick Unity editor skin"))
                    {
                        guiSkin = EditorGUIUtility.GetBuiltinSkin(EditorSkin.Inspector);
                    }

                    if (guiSkin != null)
                    {
                        var styles = guiSkin.customStyles;
                        showStyles = styles.Where(x => x.name.IndexOf(find, StringComparison.OrdinalIgnoreCase) != -1).ToArray();

                        using (gui.group_scroll(ref scrollPos, css.control_name("two")))
                        {
                            index = GUILayout.SelectionGrid(index, showStyles.Select(x => x.name).ToArray(), 1, "OL Elem");
                            if (index >= showStyles.Length)
                                index = 0;
                        }

                        using (gui.group_horizontal())
                        {
                            GUILayout.Label(showStyles[index].name);
                            if (GUILayout.Button("copy"))
                                EditorGUIUtility.systemCopyBuffer = showStyles[index].name;
                        }
                    }
                }

                using (gui.group_vertical())
                {
                    if (guiSkin != null)
                    {
                        using (gui.group_horizontal())
                        {
                            using (gui.group_vertical(css.expand_width(true)))
                            {
                                DrawStyle(x => x.normal, "normal");
                                DrawStyle(x => x.hover, "hover");
                                DrawStyle(x => x.active, "active");
                                DrawStyle(x => x.focused, "focused");
                            }
                            if (GUILayoutUtility.GetLastRect().Contains(Event.current.mousePosition))
                                gui.control_focus("two");

                            using (gui.group_vertical(css.expand_width(true)))
                            {
                                DrawStyle(x => x.onNormal, "onNormal");
                                DrawStyle(x => x.onHover, "onHover");
                                DrawStyle(x => x.onActive, "onActive");
                                DrawStyle(x => x.onFocused, "onFocused");
                            }
                            if (GUILayoutUtility.GetLastRect().Contains(Event.current.mousePosition))
                                gui.control_focus("two");
                        }

                        var methods =
                            typeof(GUIStyle).GetMethods()
                                .Where(x => x.Name.StartsWith("get_") && x.ReturnType != typeof(GUIStyleState))
                                .ToArray();
                        foreach (MethodInfo method in methods)
                            GUILayout.Label(
                                string.Format("<b>{0}</b>: {1}", method.Name.Substring(4),
                                    method.Invoke(showStyles[index], null)), new GUIStyle("label") { richText = true });
                    }
                }
            }


            if (guiSkin == null)
                return;
            {
                if (up)
                {
                    index--;
                    if (index < 0)
                        index = 0;
                    guievent.Use();
                    Repaint();
                }

                if (down)
                {
                    index++;
                    if (index >= showStyles.Length)
                        index = showStyles.Length - 1;
                    guievent.Use();
                    Repaint();
                }
            }

        }

        public void DrawStyle(Func<GUIStyle, GUIStyleState> state, string text)
        {
            if (guiSkin != null)
            {
                var style = new GUIStyle(showStyles[index]) { normal = state.Invoke(showStyles[index]) };
                GUILayout.Box(text, style);
            }
        }
    }
}