﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using CrystalMpq.DataFormats;
using UnityEditor;
using UnityEngine;
using UnityEngine.UI;
using w3model_support;
using xgm.guru.w3_to_unity.Model;
using xgm.guru.w3_to_unity.ModelFormats;

public class TestWindow : EditorWindow 
{
    [MenuItem("Window/Test Mdx")]
    public static void Open()
    {
        GetWindow<TestWindow>(true, "mdx test window", true);
    }

    public string pathForMdx = "", pathForSave = "";
    public string pathForBlp = "", pathForSaveBlp = "";
    public void OnGUI()
    {
        pathForMdx = EditorGUILayout.TextField("Path for mdx", pathForMdx);
        pathForSave = EditorGUILayout.TextField("Path for save", pathForSave);
        if (GUILayout.Button("Save"))
        {
            var fbx = AssetDatabase.LoadAllAssetsAtPath("Assets/ocean_plane.fbx");
            foreach (var o in fbx)
            {
                Debug.Log(o.GetType());
            }
            
            var obj = SaveMdxAsGameObject(pathForMdx, pathForSave);
        }

        pathForBlp = EditorGUILayout.TextField("Path for blp", pathForBlp);
        pathForSaveBlp = EditorGUILayout.TextField("Path for save", pathForSaveBlp);
        if (GUILayout.Button("Save"))
        {
            SaveBlpAsPng(pathForBlp, pathForSaveBlp);
        }
    }

    private void SaveBlpAsPng(string pathForBlp, string pathForPng)
    {
        using (var stream = new FileStream(pathForBlp, FileMode.Open))
        {
            var exampleBLP = new CrystalMpq.DataFormats.BlpTexture(stream, true);
            var tex = new Texture2D(exampleBLP.Width, exampleBLP.Height, TextureFormat.ARGB32, true);
            var data = exampleBLP.FirstMipmap.ToArray();

//            exampleBLP.FirstMipmap.CopyToArgb(new SurfaceData(tex.width, tex.height, tex.GetNativeTexturePtr(), 4)); 
            Debug.Log(data.Length);
//            tex.LoadRawTextureData(data);
             
//            var bmp = exampleBLP.FirstMipmap.CopyToArgb();
//            var bitmap = new Bitmap(texture.Width, texture.Height, PixelFormat.Format32bppArgb);
//
//            try
//            {
//                var bitmapData = bitmap.LockBits(new Rectangle(0, 0, texture.Width, texture.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
//
//                texture.FirstMipmap.CopyToArgb(new SurfaceData(bitmapData.Width, bitmapData.Height, bitmapData.Scan0, bitmapData.Stride));
//
//
//                bitmap.UnlockBits(bitmapData);
//
//                return bitmap;
//            }
//            catch { bitmap.Dispose(); throw; }
            tex.Apply();
            exampleBLP.Dispose();
            AssetDatabase.CreateAsset(tex, pathForPng);
        }
    }


    private static Mesh[] GetMdxAsSubmeshes(string path)
    {
        var mdxLoader = new CMdx();
        var model = new CModel();

        using (var stream = new FileStream(path, FileMode.Open))
        {
            mdxLoader.Load("wall", stream, model);
        }


        var submeshes = new List<Mesh>();
        foreach (var geoset in model.Geosets)
        {
            var submesh = new Mesh();
            var texturePath = geoset.Material.Object.Layers[0].Texture.Object.FileName;

            submesh.vertices = geoset.Vertices.Select(x => x.Position).ToArray();
            submesh.normals = geoset.Vertices.Select(x => x.Normal).ToArray();
            submesh.uv = geoset.Vertices.Select(x => x.TexturePosition).ToArray();

            var triangles = new List<int>();
            foreach (var face in geoset.Faces)
            {
                triangles.Add(face.Vertex1.ObjectId);
                triangles.Add(face.Vertex2.ObjectId);
                triangles.Add(face.Vertex3.ObjectId);
            }
            submesh.triangles = triangles.ToArray();
//            return submesh;
            submeshes.Add(submesh);
        }
        return submeshes.ToArray();
    }

    private static string[] GetTextures(string path)
    {
        var mdxLoader = new CMdx();
        var model = new CModel();

        using (var stream = new FileStream(path, FileMode.Open))
            mdxLoader.Load("wall", stream, model);


        var textures = new List<string>();
        foreach (var geoset in model.Geosets)
        {
            var texturePath = geoset.Material.Object.Layers[0].Texture.Object.FileName;
            if (texturePath != "" && !textures.Contains(texturePath))
                textures.Add(texturePath);
        }
        return textures.ToArray();
    }

    public static Mesh SaveMdxAsMesh(string pathForMdx, string pathForMesh)
    {
        var mesh = new Mesh();
        var submeshes = GetMdxAsSubmeshes(pathForMdx);
        mesh.CombineMeshes(submeshes.Select(x => new CombineInstance { mesh = x, transform = GetMatrix() }).ToArray(), false, true);
        AssetDatabase.CreateAsset(mesh, pathForMesh);
        foreach (var submesh in submeshes)
            AssetDatabase.AddObjectToAsset(submesh, mesh);
        return mesh;
    }

    public static GameObject SaveMdxAsGameObject(string pathForMdx, string pathForMesh)
    {
        var go = new GameObject(Path.GetFileNameWithoutExtension(pathForMesh));
        go.AddComponent<MeshFilter>();
        go.AddComponent<MeshRenderer>();
        const string extension = ".prefab";
        PrefabUtility.CreatePrefab(pathForMesh + extension, go);
        var g = AssetDatabase.LoadAssetAtPath(pathForMesh + extension, typeof(GameObject)) as GameObject;
        var mf = g.GetComponent<MeshFilter>();
        mf.sharedMesh = new Mesh();
        var submeshes = GetMdxAsSubmeshes(pathForMdx);
        mf.sharedMesh.CombineMeshes(submeshes.Select(x => new CombineInstance { mesh = x, transform = GetMatrix() }).ToArray(), false, true);
        var mr = g.GetComponent<MeshRenderer>();
        mr.sharedMaterials = new Material[submeshes.Length];
        mf.sharedMesh.name = Path.GetFileNameWithoutExtension(pathForMesh);
        AssetDatabase.CreateAsset(mf.sharedMesh, pathForMesh + ".asset");
//        AssetDatabase.AddObjectToAsset(mf.sharedMesh, g);
        DestroyImmediate(go);
        
//        AssetDatabase.AddObjectToAsset(mf.sharedMesh, go);
//        foreach (var submesh in submeshes)
//            AssetDatabase.AddObjectToAsset(submesh, go);
//        DestroyImmediate(go, false);
//        AssetDatabase.Refresh();
//        AssetDatabase.RenameAsset(pathForMesh + ".prefab", pathForMesh);
//        AssetDatabase.Refresh();
//        foreach (var submesh in submeshes)
//            AssetDatabase.AddObjectToAsset(submesh, mesh);
        return go;
    }

    public static Mesh GetMdxAsMesh(string pathForMdx)
    {
        var mesh = new Mesh();
        var submeshes = GetMdxAsSubmeshes(pathForMdx);
        mesh.CombineMeshes(submeshes.Select(x => new CombineInstance { mesh = x, transform = GetMatrix() }).ToArray(), false, true);
        return mesh;
    }

    private static Matrix4x4 GetMatrix()
    {
        var matrix = new Matrix4x4();
        matrix.SetTRS(Vector3.zero, Quaternion.Euler(-90, 0, 0), new Vector3(1, 1, 1));
        return matrix;
    }
}
