﻿using System.Collections.Generic;
using System.IO;
using UnityEditor;
using UnityEditor.AnimatedValues;
using UnityEngine;
using xgm.guru.w3_to_unity.Model;
using xgm.guru.w3_to_unity.ModelFormats;

public class W3ImporterWindow : EditorWindow
{
    [MenuItem("Window/W3Importer")]
    public static void Open()
    {
        GetWindow<W3ImporterWindow>(true, "Warcraft III Importer", true).Init("Assets/PhoenixSwordsman.mdx");
    }

    public AnimBool animWarcraft;
    public AnimBool animTextures;
    public AnimBool animAnimations;
    public Vector2 scrollTextureList;
    public List<string> textureList = new List<string>();
    public string pathForMdx;
    public string modelName;
    public int submeshCount;

    public void Init(string pathForMdx)
    {
        this.pathForMdx = pathForMdx;
        var model = GetCModel(pathForMdx);
        textureList = GetTextures(model);
        modelName = Path.GetFileName(pathForMdx);
        submeshCount = model.Geosets.Count;
    }
    
    private static List<string> GetTextures(CModel model)
    {
        var textures = new List<string>();
        foreach (var geoset in model.Geosets)
        {
            var texturePath = geoset.Material.Object.Layers[0].Texture.Object.FileName;
            if (texturePath != "" && !textures.Contains(texturePath))
                textures.Add(texturePath);
        }
        return textures;
    }
    
    private static CModel GetCModel(string path)
    {
        var mdxLoader = new CMdx();
        var model = new CModel();

        using (var stream = new FileStream(path, FileMode.Open))
            mdxLoader.Load("wall", stream, model);

        return model;
    }

//    public static GameObject SaveMdxAsGameObject(string pathForMdx, string pathForMesh)
//    {
//        var go = new GameObject(Path.GetFileNameWithoutExtension(pathForMesh));
//        go.AddComponent<MeshFilter>();
//        go.AddComponent<MeshRenderer>();
//        const string extension = ".prefab";
//        PrefabUtility.CreatePrefab(pathForMesh + extension, go);
//        var g = AssetDatabase.LoadAssetAtPath(pathForMesh + extension, typeof(GameObject)) as GameObject;
//        var mf = g.GetComponent<MeshFilter>();
//        mf.sharedMesh = new Mesh();
//        var submeshes = GetMdxAsSubmeshes(pathForMdx);
//        mf.sharedMesh.CombineMeshes(submeshes.Select(x => new CombineInstance { mesh = x, transform = GetMatrix() }).ToArray(), false, true);
//        var mr = g.GetComponent<MeshRenderer>();
//        mr.sharedMaterials = new Material[submeshes.Length];
//        mf.sharedMesh.name = Path.GetFileNameWithoutExtension(pathForMesh);
//        AssetDatabase.CreateAsset(mf.sharedMesh, pathForMesh + ".asset");
//        //        AssetDatabase.AddObjectToAsset(mf.sharedMesh, g);
//        DestroyImmediate(go);
//
//        //        AssetDatabase.AddObjectToAsset(mf.sharedMesh, go);
//        //        foreach (var submesh in submeshes)
//        //            AssetDatabase.AddObjectToAsset(submesh, go);
//        //        DestroyImmediate(go, false);
//        //        AssetDatabase.Refresh();
//        //        AssetDatabase.RenameAsset(pathForMesh + ".prefab", pathForMesh);
//        //        AssetDatabase.Refresh();
//        //        foreach (var submesh in submeshes)
//        //            AssetDatabase.AddObjectToAsset(submesh, mesh);
//        return go;
//    }

    void OnEnable()
    {
        animWarcraft = new AnimBool(true);
        animTextures = new AnimBool(true);
        animAnimations = new AnimBool(true);
        animWarcraft.valueChanged.AddListener(Repaint);
        animAnimations.valueChanged.AddListener(Repaint);
        animTextures.valueChanged.AddListener(Repaint);
    }

    public void OnGUI()
    {
        GUILayout.Label(modelName, "PreButton");

        if (Title("Paths", animWarcraft))
        {
            var l1 = new GUIContent("Warcraft 3 path",
                "Путь до папки Warcraft 3 на случай необходимости импортировать текстуры оттуда");
            GUILayout.BeginHorizontal();
            EditorGUILayout.TextField(l1, "");
            if (GUILayout.Button("...", "minibutton", GUILayout.ExpandWidth(false)))
                EditorUtility.OpenFolderPanel("Директория Warcraft III", "", "");
            GUILayout.EndHorizontal();
        }
        End();

        if (Title("Textures", animTextures))
        {
            var l1 = new GUIContent("Список текстур",
                "Список текстур, используемых в модели");
            EditorGUILayout.PrefixLabel(l1);

            GUILayout.BeginVertical("ProgressBarBack");
            scrollTextureList = GUILayout.BeginScrollView(scrollTextureList, "TL LogicBar parentgrey", GUILayout.Height(100));
            for (int i = 0; i < textureList.Count; i++)
                GUILayout.Label(textureList[i], "HelpBox");
            GUILayout.EndScrollView();
            GUILayout.EndVertical();

            if (submeshCount > textureList.Count)
            {
                GUILayout.Label(string.Format("Указанные текстуры имеют {0} ссылок в модели. Это больше, чем реальное количество текстур и может позволить вам использовать уникальные материалы в каждом отдельном случае, однако, создаст дополнительные файлы.",
                        submeshCount), new GUIStyle("label") {wordWrap = true});
                GUILayout.SelectionGrid(0, new[] {"Использовать уникальные материалы", "Использовать общие материалы"},
                    1, "Radio");
            }
        }
        End();
        if (Title("Animations", animAnimations))
        {
        }
        End();
        GUILayout.FlexibleSpace();
        GUILayout.BeginVertical("ProgressBarBack");
        EditorGUILayout.ToggleLeft("Использовать аналогичные настройки для всех следующих файлов в очереди?", false);
        GUILayout.BeginHorizontal();
        GUILayout.Button("OK");
        GUILayout.Button("Отмена");
        GUILayout.EndHorizontal();
        GUILayout.EndVertical();
    }

    private static bool Title(string name, AnimBool ab)
    {
        ab.target = GUILayout.Toggle(ab.target, new GUIContent(name), "IN Title");
        var result = EditorGUILayout.BeginFadeGroup(ab.faded);
        if (result)
            EditorGUI.indentLevel++;
        return result;
    }

    private static void End()
    {
        EditorGUI.indentLevel = 0;
        EditorGUILayout.EndFadeGroup();
    }
}