﻿using UnityEngine;

public sealed class Extent : System.ICloneable
{
    /// <summary>
    /// Default constructor.
    /// </summary>
    public Extent()
    {
        _min = new Vector3();
        _max = new Vector3();
    }

    /// <summary>
    /// Copy constructor.
    /// </summary>
    /// <param name="extent">The extent to copy from</param>
    public Extent(Extent extent)
    {
        _min = extent._min;
        _max = extent._max;
        _radius = extent._radius;
    }

    /// <summary>
    /// Parameterized constructor.
    /// </summary>
    /// <param name="min">The minimum point to use</param>
    /// <param name="max">The maximum point to use</param>
    /// <param name="radius">The radius to use</param>
    public Extent(Vector3 min, Vector3 max, float radius)
    {
        _min = min;
        _max = max;
        _radius = radius;
    }

    public object Clone()
        { return new Extent(this); }

    public override string ToString()
        { return "{ " + _min + ", " + _max + ", " + _radius + " }"; }

    /// <summary> Retrieves the minimum point. </summary>
    public Vector3 min { get { return _min; } }
    private Vector3 _min;

    /// <summary> Retrieves the maximum point. </summary>
    public Vector3 max { get { return _max; } }
    private Vector3 _max;

    /// <summary> Retrieves the radius. </summary>
    public float radius { get { return _radius; } }
    private float _radius;
}