﻿using System.IO;

namespace xgm.guru.w3_to_unity
{
    internal sealed class Mdx
    {
        /// <summary>
        /// Loads a model from a stream.
        /// </summary>
        /// <param name="Name">The name of the model (only used in some error messages)</param>
        /// <param name="Stream">The stream to load from (must support reading)</param>
        /// <param name="model">The model to load to (must be an empty model)</param>
        internal void Load(string Name, Stream Stream, MdxModel model)
        {
            if (!Stream.CanRead) 
                throw new System.NotSupportedException("Unable to load \"" + Name + "\", the stream does not support reading!");

            var reader = new MdxReader(Name, Stream);
            MdxModel.instance.Load(reader, model);
            reader.Attacher.Attach();
        }

        /// <summary>
        /// Saves a model to a stream.
        /// </summary>
        /// <param name="Name">The name of the model (only used in some error messages)</param>
        /// <param name="Stream">The stream to save to (must support writing and seeking)</param>
        /// <param name="Model">The model to save from</param>
        public void Save(string Name, Stream Stream, MdxModel.CModel Model)
        {
            if (!Stream.CanWrite) throw new System.NotSupportedException("Unable to load \"" + Name + "\", the stream does not support writing!");
            if (!Stream.CanSeek) throw new System.NotSupportedException("Unable to save \"" + Name + "\", the stream does not support seeking!");

            Mdx.CSaver Saver = new Mdx.CSaver(Name, Stream);
            Mdx.CModel.Instance.Save(Saver, Model, BuildHeader(Name));
        }

        private string BuildHeader(string Name)
        {
            System.Text.StringBuilder Header = new System.Text.StringBuilder();

            Header.Append(Name.Replace("\n", "").Replace("\r", ""));
            Header.Append(", ");
            Header.Append(CConstants.HeaderFullName);
            Header.Append(", ");
            Header.Append(System.DateTime.Now.ToString(CConstants.HeaderDateFormat));
            Header.Append(", ");
            Header.Append(CConstants.HeaderUrl);

            return Header.ToString();
        }
    }
}
