﻿using System.Collections.Generic;
using System.IO;
using UnityEngine;

namespace xgm.guru.w3_to_unity
{
    /// <summary> Считывает файл mdx </summary>
    internal sealed class MdxReader
    {
        public string name { get; private set; }

        private BinaryReader binaryReader;
        public long position { get { return binaryReader.BaseStream.Position; } }
        public bool eof { get { return (binaryReader.BaseStream.Position >= binaryReader.BaseStream.Length); } }

        private LinkedList<int> locations;

        public Attacher.CAttacherContainer attacher { get; private set; }
        
        public MdxReader(string name, Stream stream)
        {
            this.name = name;
            attacher = new Attacher.CAttacherContainer();
            binaryReader = new BinaryReader(stream, MdxConsts.SimpleTextEncoding);
            locations = new LinkedList<int>();
        }
        
        public byte[] ReadBytes(int size)
            { return binaryReader.ReadBytes(size); }

        public byte ReadByte()
            { return binaryReader.ReadByte(); }

        public int ReadInt8()
            { return binaryReader.ReadByte(); }

        public int ReadInt16()
            { return binaryReader.ReadInt16(); }

        public int ReadInt32()
            { return binaryReader.ReadInt32(); }

        public float ReadFloat()
            { return binaryReader.ReadSingle(); }

        public double ReadDouble()
            { return binaryReader.ReadDouble(); }

        public string ReadString(int length)
        {
            int BufferLength = length;
            char[] Buffer = binaryReader.ReadChars(length);

            while (BufferLength > 0)
            {
                if (Buffer[BufferLength - 1] != '\0') break;
                BufferLength--;
            }

            return new string(Buffer, 0, BufferLength);
        }

        public string ReadTag()
        {
            return ReadString(MdxConsts.SizeTag);
        }

        public Vector2 ReadVector2()
        {
            var x = ReadFloat();
            var y = ReadFloat();
            return new Vector2(x, y);
        }

        public Vector3 ReadVector3()
        {
            var x = ReadFloat();
            var y = ReadFloat();
            var z = ReadFloat();
            return new Vector3(x, y, z);
        }

        public Vector4 ReadVector4()
        {
            var w = ReadFloat();
            var x = ReadFloat();
            var y = ReadFloat();
            var z = ReadFloat();
            return new Vector4(w, x, y, z);
        }

        public Quaternion ReadQuaternion()
        {
            var x = ReadFloat();
            var y = ReadFloat();
            var z = ReadFloat();
            var w = ReadFloat();
            return new Quaternion(x, y, z, w);
        }

        public Extent ReadExtent()
        {
            var radius = ReadFloat();
            var min = ReadVector3();
            var max = ReadVector3();
            return new Extent(min, max, radius);
        }

        public void ExpectTag(string expectedTag)
        {
            var tag = ReadTag();
            if (tag != expectedTag)
                throw new System.Exception(
                    string.Format("Error at location {0}, expected \"{1}\", got \"{2}\"!",
                        position, expectedTag, tag));
        }

        public void Skip(int nrOfBytes)
        {
            binaryReader.ReadBytes(nrOfBytes);
        }

        public void PushLocation()
        {
            locations.AddLast((int)position);
        }

        public int PopLocation()
        {
            var location = locations.Last.Value;
            locations.RemoveLast();
            return (int)position - location;
        }


    }
}